"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Rx = require("rxjs");
exports.State = {
    OK: 'OK',
    ERROR: 'ERROR'
};
var QEntry = /** @class */ (function () {
    function QEntry(task) {
        this.executeCnt = 0;
        this.task = task;
        this.created = new Date();
    }
    return QEntry;
}());
exports.QEntry = QEntry;
var Logger = /** @class */ (function () {
    function Logger(upStream) {
        this.upStream = upStream;
    }
    Logger.prototype.info = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this.upStream.next({ level: 'info', parts: args });
    };
    Logger.prototype.error = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this.upStream.next({ level: 'error', parts: args });
    };
    Logger.prototype.debug = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this.upStream.next({ level: 'debug', parts: args });
    };
    return Logger;
}());
var Queue = /** @class */ (function () {
    function Queue(logger, argv) {
        // public q: Rx.Observable<Entry>;
        this.qEntries = [];
        this.inProcess = false;
        this.taskTimer = argv.taskTimer || 500;
        this.reclaimTimeout = argv.reclaimTimeout || 10000;
        this.retryWaitTime = argv.retryWaitTime || 1000;
        this.maxExecuteCnt = argv.maxExecuteCnt || 10;
        this.logger = new Logger(logger);
        this.q = new Rx.Subject();
        this.deadLetter = new Rx.Subject();
        this.q.subscribe(this.action.bind(this));
        this.qTask = setInterval(this.processMemoryQ.bind(this), this.taskTimer);
    }
    Queue.prototype.stop = function () {
        var _this = this;
        this.logger.info('run stop');
        // console.log('run stop');
        return Rx.Observable.create(function (observer) {
            _this.logger.info('queue stop');
            var action = function () {
                if (_this.qEntries.length == 0) {
                    _this.logger.info('Q Task stopped');
                    clearInterval(_this.qTask);
                    observer.next(null);
                    observer.complete();
                }
                else {
                    _this.logger.info('waiting for Q Task to stop', _this.qEntries.length);
                    setTimeout(action, _this.taskTimer);
                }
            };
            action();
        });
    };
    Queue.prototype.action = function (qe) {
        var _this = this;
        this.logger.debug('action:', qe);
        qe.running = new Date();
        ++qe.executeCnt;
        qe.task.subscribe(function (_qe) {
            qe.running = null;
            qe.completed = new Date();
            _this.logger.debug('action:completed:', qe);
        }, function (error) {
            qe.running = null;
            qe.retryAt = new Date((new Date()).getTime() + _this.retryWaitTime);
            _this.logger.debug('action:error:', qe.created.getTime(), qe.retryAt.getTime(), qe);
        });
    };
    Queue.prototype.processMemoryQ = function () {
        var _this = this;
        if (this.inProcess) {
            return;
        }
        this.inProcess = true;
        var now = (new Date()).getTime();
        this.qEntries = this.qEntries.filter(function (qe) {
            if (qe.completed) {
                _this.logger.debug('completed remove', qe);
                return false;
            }
            else if (qe.executeCnt >= _this.maxExecuteCnt) {
                _this.logger.error('drop queue entry:', qe);
                _this.deadLetter.next(qe);
                return false;
            }
            else if (qe.retryAt) {
                if (now >= qe.retryAt.getTime()) {
                    qe.retryAt = null;
                    _this.logger.debug('retry queue entry:', qe);
                    _this.q.next(qe);
                }
                return true;
            }
            else if (!qe.running) {
                _this.logger.debug('trigger action:', qe);
                _this.q.next(qe);
            }
            else if ((qe.running && qe.executeCnt < _this.maxExecuteCnt &&
                (now - qe.running.getTime()) >= _this.reclaimTimeout)) {
                qe.running = null;
                _this.logger.debug('retry execution time entry:', qe);
                _this.q.next(qe);
            }
            return true;
        });
        this.inProcess = false;
    };
    Queue.prototype.push = function (action) {
        this.qEntries.push(new QEntry(action));
        this.processMemoryQ();
    };
    return Queue;
}());
exports.Queue = Queue;
function start(logger, argv) {
    return new Queue(logger, argv);
}
exports.start = start;
exports.default = Queue;
//# sourceMappingURL=simqle.js.map